<?php

declare(strict_types=1);

namespace Drupal\search_api_field_datasource\Plugin\search_api\datasource;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\search_api_field_datasource\Plugin\search_api\datasource\Hacks\ContentEntityHack;

/**
 * Represents a datasource which exposes the content entities.
 *
 * @SearchApiDatasource(
 *   id = "entity_field_key",
 *   deriver = "Drupal\search_api_field_datasource\Plugin\Deriver\ContentEntityDeriver"
 * )
 */
class FieldKeyedContentEntity extends ContentEntityHack implements PluginFormInterface, ContentEntityDatasourceInterface {

  /**
   * {@inheritDoc}
   */
  #[\Override]
  public function defaultConfiguration(): array {
    $config = parent::defaultConfiguration();
    $config['key_field'] = '';
    return $config;
  }

  /**
   * {@inheritDoc}
   */
  #[\Override]
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    // @todo use entity system to get a list of valid fields for the entity type?
    $form['key_field'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Field name'),
      '#default_value' => $this->configuration['key_field'],
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function loadMultiple(array $ids) {
    $allowed_languages = $this->getLanguages();
    $field_key = $this->getConfiguration()['key_field'];
    if ($field_key == 'label') {
      $field_key = $this->getEntityType()->getKey('label');
    }

    $field_values = [];
    foreach ($ids as $item_id) {
      $pos = strrpos($item_id, ':');
      // This can only happen if someone passes an invalid ID, since we always
      // include a language code. Still, no harm in guarding against bad input.
      if ($pos === FALSE) {
        continue;
      }
      $field_value = substr($item_id, 0, $pos);
      $langcode = substr($item_id, $pos + 1);
      if (isset($allowed_languages[$langcode])) {
        $field_values[$field_value][$item_id] = $langcode;
      }
    }

    /** @var \Drupal\Core\Entity\ContentEntityInterface[] $entities */
    $entities = $this->getEntityStorage()->loadByProperties([
      $field_key => array_keys($field_values),
    ]);

    $items = [];
    $allowed_bundles = $this->getBundles();
    foreach ($entities as $entity_id => $entity) {
      if (isset($allowed_bundles[$entities[$entity_id]->bundle()])) {
        $field_value = $entity->get($field_key)->value;
        foreach ($field_values[$field_value] as $item_id => $langcode) {
          $items[$item_id] = $entity->getTranslation($langcode)->getTypedData();
        }
      }
    }

    return $items;
  }

  /**
   * {@inheritDoc}
   */
  #[\Override]
  public function getEntityTrackingId(EntityInterface $entity, string $langcode): string {
    $key_field = $this->getConfiguration()['key_field'];
    if (empty($key_field) || $key_field == 'id') {
      $field = $entity->id();
    }
    elseif ($key_field == 'url') {
      $field = $entity->toUrl('canonical');
    }
    elseif ($key_field == 'label') {
      $field = $entity->label();
    }
    else {
      assert($entity instanceof ContentEntityInterface);
      $field = $entity->get($key_field)->value;
    }
    if ($field) {
      return $field . ':' . $langcode;
    }
    // This will be treated as invalid and excluded which will avoid duplicate
    // constraint errors for unpopulated fields.
    return '';
  }

}
