<?php

declare(strict_types=1);

namespace Drupal\search_api_field_datasource\Plugin\search_api\datasource\Hacks;

use Drupal\Component\Utility\DeprecationHelper;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Plugin\search_api\datasource\ContentEntityTrackingManager;
use Drupal\search_api_field_datasource\Plugin\search_api\datasource\ContentEntityDatasourceInterface;

/**
 * Hacks to search_api's tracking manager to generalize it.
 *
 * Move some of its logic around so the tracking ID can be generated by the
 * data source plugin instead of hardcoding that logic into this.
 *
 * @see \Drupal\search_api_field_datasource\Plugin\search_api\datasource\ContentEntityDatasourceInterface
 */
abstract class ContentEntityTrackingManagerHacks extends ContentEntityTrackingManager {

  /**
   * {@inheritDoc}
   */
  public function trackEntityChange(ContentEntityInterface $entity, bool $new = FALSE): void {
    // Check if the entity is a content entity.
    if (!empty($entity->search_api_skip_tracking)) {
      return;
    }

    $indexes = $this->getIndexesForEntity($entity);
    if (!$indexes) {
      return;
    }

    // Compare old and new languages for the entity to identify inserted,
    // updated and deleted translations (and, therefore, search items).
    $new_translations = array_keys($entity->getTranslationLanguages());
    $old_translations = [];
    if (!$new) {
      // In case we don't have the original, fall back to the current entity,
      // and assume no new translations were added.
      $original = DeprecationHelper::backwardsCompatibleCall(
        \Drupal::VERSION,
        '11.2',
        fn () => $entity->getOriginal() ?: $entity,
        fn () => $entity->original ?? $entity,
      );
      $old_translations = array_keys($original->getTranslationLanguages());
    }
    $deleted_translations = array_diff($old_translations, $new_translations);
    $inserted_translations = array_diff($new_translations, $old_translations);
    $updated_translations = array_diff($new_translations, $inserted_translations);

    $datasource_id = static::DATASOURCE_BASE_ID . ':' . $entity->getEntityTypeId();

    // NOTE: These lines changed to call class method with entity instead of
    // wrapped static call. Also moved into the loop so it can be unique to each
    // index's config.
    foreach ($indexes as $index) {
      $get_ids = fn ($langcode) => $this->getEntityTrackingId($index, $entity, $langcode);
      $inserted_ids = array_map($get_ids, $inserted_translations);
      $updated_ids = array_map($get_ids, $updated_translations);
      $deleted_ids = array_map($get_ids, $deleted_translations);
      if ($inserted_ids) {
        $filtered_item_ids = static::filterValidItemIds($index, $datasource_id, $inserted_ids);
        if ($filtered_item_ids) {
          $index->trackItemsInserted($datasource_id, $filtered_item_ids);
        }
      }
      if ($updated_ids) {
        $filtered_item_ids = static::filterValidItemIds($index, $datasource_id, $updated_ids);
        if ($filtered_item_ids) {
          $index->trackItemsUpdated($datasource_id, $filtered_item_ids);
        }
      }
      if ($deleted_ids) {
        $filtered_item_ids = static::filterValidItemIds($index, $datasource_id, $deleted_ids);
        if ($filtered_item_ids) {
          $index->trackItemsDeleted($datasource_id, $filtered_item_ids);
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public function entityDelete(EntityInterface $entity): void {
    // Check if the entity is a content entity.
    if (!($entity instanceof ContentEntityInterface)
      || !empty($entity->search_api_skip_tracking)) {
      return;
    }

    $indexes = $this->getIndexesForEntity($entity);
    if (!$indexes) {
      return;
    }

    // Remove the search items for all the entity's translations.
    $item_ids = [];
    $datasource_id = static::DATASOURCE_BASE_ID . ':' . $entity->getEntityTypeId();
    foreach ($indexes as $index) {
      foreach (array_keys($entity->getTranslationLanguages()) as $langcode) {
        $item_ids[] = $this->getEntityTrackingId($index, $entity, $langcode);
      }
      $index->trackItemsDeleted($datasource_id, $item_ids);
    }
  }

  /**
   * Get the entity id used for tracking.
   *
   * @param \Drupal\search_api\IndexInterface $index
   *   Index being considered.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity being tracked.
   * @param string $langcode
   *   Language code of the entity being tracked.
   *
   * @return string
   *   Tracking id.
   */
  protected function getEntityTrackingId(IndexInterface $index, EntityInterface $entity, string $langcode): string {
    $ds = $index->getDatasource(static::DATASOURCE_BASE_ID . ':' . $entity->getEntityTypeId());
    if ($ds instanceof ContentEntityDatasourceInterface) {
      return $ds->getEntityTrackingId($entity, $langcode);
    }
    return static::formatItemId($entity->getEntityTypeId(), $entity->id(), $langcode);
  }

}
